@@ -159,6 +159,20 @@ pub(crate) fn read_i64<R: Read + ?Sized>(reader: &mut R) -> Result<i64> {
     Ok(i64::from_le_bytes(buf))
 }
 
+#[inline]
+pub(crate) fn read_u32<R: Read + ?Sized>(reader: &mut R) -> Result<u32> {
+    let mut buf = [0; 4];
+    reader.read_exact(&mut buf)?;
+    Ok(u32::from_le_bytes(buf))
+}
+
+#[inline]
+pub(crate) fn read_u64<R: Read + ?Sized>(reader: &mut R) -> Result<u64> {
+    let mut buf = [0; 8];
+    reader.read_exact(&mut buf)?;
+    Ok(u64::from_le_bytes(buf))
+}
+
 #[inline]
 fn read_f64<R: Read + ?Sized>(reader: &mut R) -> Result<f64> {
     let mut buf = [0; 8];
@@ -245,6 +259,8 @@ pub(crate) fn deserialize_bson_kvp<R: Read + ?Sized>(
         }
         Some(ElementType::Int32) => read_i32(reader).map(Bson::Int32)?,
         Some(ElementType::Int64) => read_i64(reader).map(Bson::Int64)?,
+        Some(ElementType::UInt32) => read_u32(reader).map(Bson::UInt32)?,
+        Some(ElementType::UInt64) => read_u64(reader).map(Bson::UInt64)?,
         Some(ElementType::Timestamp) => Bson::Timestamp(Timestamp::from_reader(reader)?),
         Some(ElementType::DateTime) => {
             // The int64 is UTC milliseconds since the Unix epoch.
