@@ -6,8 +6,8 @@ use std::{
 use serde::{ser::SerializeMap, Deserialize, Serialize};
 
 use crate::{
-    de::MIN_BSON_DOCUMENT_SIZE,
-    raw::{error::ErrorKind, serde::OwnedOrBorrowedRawDocument, RAW_DOCUMENT_NEWTYPE},
+    raw::{error::ErrorKind, RawBsonVisitor, RAW_DOCUMENT_NEWTYPE},
+    spec::BinarySubtype,
     DateTime,
     Timestamp,
 };
@@ -18,10 +18,10 @@ use super::{
     Error,
     Iter,
     RawArray,
-    RawBinaryRef,
-    RawBsonRef,
+    RawBinary,
+    RawBson,
     RawDocumentBuf,
-    RawRegexRef,
+    RawRegex,
     Result,
 };
 use crate::{oid::ObjectId, spec::ElementType, Document};
@@ -43,7 +43,7 @@ use crate::{oid::ObjectId, spec::ElementType, Document};
 /// # use bson::raw::{Error};
 /// use bson::raw::RawDocument;
 ///
-/// let doc = RawDocument::from_bytes(b"\x13\x00\x00\x00\x02hi\x00\x06\x00\x00\x00y'all\x00\x00")?;
+/// let doc = RawDocument::new(b"\x13\x00\x00\x00\x02hi\x00\x06\x00\x00\x00y'all\x00\x00")?;
 /// let mut iter = doc.into_iter();
 /// let (key, value) = iter.next().unwrap()?;
 /// assert_eq!(key, "hi");
@@ -60,7 +60,7 @@ use crate::{oid::ObjectId, spec::ElementType, Document};
 /// ```
 /// use bson::raw::RawDocument;
 ///
-/// let doc = RawDocument::from_bytes(b"\x13\x00\x00\x00\x02hi\x00\x06\x00\x00\x00y'all\x00\x00")?;
+/// let doc = RawDocument::new(b"\x13\x00\x00\x00\x02hi\x00\x06\x00\x00\x00y'all\x00\x00")?;
 /// assert_eq!(doc.get_str("hi")?, "y'all");
 /// # Ok::<(), Box<dyn std::error::Error>>(())
 /// ```
@@ -86,10 +86,10 @@ impl RawDocument {
     /// ```
     /// use bson::raw::RawDocument;
     ///
-    /// let doc = RawDocument::from_bytes(b"\x05\0\0\0\0")?;
+    /// let doc = RawDocument::new(b"\x05\0\0\0\0")?;
     /// # Ok::<(), bson::raw::Error>(())
     /// ```
-    pub fn from_bytes<D: AsRef<[u8]> + ?Sized>(data: &D) -> Result<&RawDocument> {
+    pub fn new<D: AsRef<[u8]> + ?Sized>(data: &D) -> Result<&RawDocument> {
         let data = data.as_ref();
 
         if data.len() < 5 {
@@ -138,18 +138,18 @@ impl RawDocument {
         unsafe { &*(data.as_ref() as *const [u8] as *const RawDocument) }
     }
 
-    /// Creates a new [`RawDocumentBuf`] with an owned copy of the BSON bytes.
+    /// Creates a new [`RawDocument`] with an owned copy of the BSON bytes.
     ///
     /// ```
     /// use bson::raw::{RawDocument, RawDocumentBuf, Error};
     ///
     /// let data = b"\x05\0\0\0\0";
-    /// let doc_ref = RawDocument::from_bytes(data)?;
+    /// let doc_ref = RawDocument::new(data)?;
     /// let doc: RawDocumentBuf = doc_ref.to_raw_document_buf();
     /// # Ok::<(), Error>(())
     pub fn to_raw_document_buf(&self) -> RawDocumentBuf {
         // unwrap is ok here because we already verified the bytes in `RawDocumentRef::new`
-        RawDocumentBuf::from_bytes(self.data.to_owned()).unwrap()
+        RawDocumentBuf::new(self.data.to_owned()).unwrap()
     }
 
     /// Gets a reference to the value corresponding to the given key by iterating until the key is
@@ -157,19 +157,19 @@ impl RawDocument {
     ///
     /// ```
     /// # use bson::raw::Error;
-    /// use bson::{rawdoc, oid::ObjectId};
+    /// use bson::{doc, oid::ObjectId, raw::{RawDocumentBuf, RawBson}};
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "_id": ObjectId::new(),
     ///     "f64": 2.5,
-    /// };
+    /// })?;
     ///
     /// let element = doc.get("f64")?.expect("finding key f64");
     /// assert_eq!(element.as_f64(), Some(2.5));
     /// assert!(doc.get("unknown")?.is_none());
     /// # Ok::<(), Error>(())
     /// ```
-    pub fn get(&self, key: impl AsRef<str>) -> Result<Option<RawBsonRef<'_>>> {
+    pub fn get(&self, key: impl AsRef<str>) -> Result<Option<RawBson<'_>>> {
         for result in self.into_iter() {
             let (k, v) = result?;
             if key.as_ref() == k {
@@ -183,7 +183,7 @@ impl RawDocument {
         &'a self,
         key: impl AsRef<str>,
         expected_type: ElementType,
-        f: impl FnOnce(RawBsonRef<'a>) -> Option<T>,
+        f: impl FnOnce(RawBson<'a>) -> Option<T>,
     ) -> ValueAccessResult<T> {
         let key = key.as_ref();
 
@@ -214,13 +214,13 @@ impl RawDocument {
     ///
     /// ```
     /// # use bson::raw::Error;
-    /// use bson::raw::ValueAccessErrorKind;
-    /// use bson::rawdoc;
+    /// use bson::raw::{ValueAccessErrorKind, RawDocumentBuf};
+    /// use bson::doc;
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "bool": true,
     ///     "f64": 2.5,
-    /// };
+    /// })?;
     ///
     /// assert_eq!(doc.get_f64("f64")?, 2.5);
     /// assert!(matches!(doc.get_f64("bool").unwrap_err().kind, ValueAccessErrorKind::UnexpectedType { .. }));
@@ -228,19 +228,19 @@ impl RawDocument {
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn get_f64(&self, key: impl AsRef<str>) -> ValueAccessResult<f64> {
-        self.get_with(key, ElementType::Double, RawBsonRef::as_f64)
+        self.get_with(key, ElementType::Double, RawBson::as_f64)
     }
 
     /// Gets a reference to the string value corresponding to a given key or returns an error if the
     /// key corresponds to a value which isn't a string.
     ///
     /// ```
-    /// use bson::{rawdoc, raw::ValueAccessErrorKind};
+    /// use bson::{doc, raw::{RawDocumentBuf, ValueAccessErrorKind}};
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "string": "hello",
     ///     "bool": true,
-    /// };
+    /// })?;
     ///
     /// assert_eq!(doc.get_str("string")?, "hello");
     /// assert!(matches!(doc.get_str("bool").unwrap_err().kind, ValueAccessErrorKind::UnexpectedType { .. }));
@@ -248,7 +248,7 @@ impl RawDocument {
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn get_str(&self, key: impl AsRef<str>) -> ValueAccessResult<&'_ str> {
-        self.get_with(key, ElementType::String, RawBsonRef::as_str)
+        self.get_with(key, ElementType::String, RawBson::as_str)
     }
 
     /// Gets a reference to the document value corresponding to a given key or returns an error if
@@ -256,12 +256,12 @@ impl RawDocument {
     ///
     /// ```
     /// # use bson::raw::Error;
-    /// use bson::{rawdoc, raw::ValueAccessErrorKind};
+    /// use bson::{doc, raw::{ValueAccessErrorKind, RawDocumentBuf}};
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "doc": { "key": "value"},
     ///     "bool": true,
-    /// };
+    /// })?;
     ///
     /// assert_eq!(doc.get_document("doc")?.get_str("key")?, "value");
     /// assert!(matches!(doc.get_document("bool").unwrap_err().kind, ValueAccessErrorKind::UnexpectedType { .. }));
@@ -269,19 +269,19 @@ impl RawDocument {
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn get_document(&self, key: impl AsRef<str>) -> ValueAccessResult<&'_ RawDocument> {
-        self.get_with(key, ElementType::EmbeddedDocument, RawBsonRef::as_document)
+        self.get_with(key, ElementType::EmbeddedDocument, RawBson::as_document)
     }
 
     /// Gets a reference to the array value corresponding to a given key or returns an error if
     /// the key corresponds to a value which isn't an array.
     ///
     /// ```
-    /// use bson::{rawdoc, raw::ValueAccessErrorKind};
+    /// use bson::{doc, raw::{RawDocumentBuf, ValueAccessErrorKind}};
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "array": [true, 3],
     ///     "bool": true,
-    /// };
+    /// })?;
     ///
     /// let mut arr_iter = doc.get_array("array")?.into_iter();
     /// let _: bool = arr_iter.next().unwrap()?.as_bool().unwrap();
@@ -293,7 +293,7 @@ impl RawDocument {
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn get_array(&self, key: impl AsRef<str>) -> ValueAccessResult<&'_ RawArray> {
-        self.get_with(key, ElementType::Array, RawBsonRef::as_array)
+        self.get_with(key, ElementType::Array, RawBson::as_array)
     }
 
     /// Gets a reference to the BSON binary value corresponding to a given key or returns an error
@@ -301,24 +301,24 @@ impl RawDocument {
     ///
     /// ```
     /// use bson::{
-    ///     rawdoc,
-    ///     raw::ValueAccessErrorKind,
+    ///     doc,
+    ///     raw::{ValueAccessErrorKind, RawDocumentBuf, RawBinary},
     ///     spec::BinarySubtype,
     ///     Binary,
     /// };
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "binary": Binary { subtype: BinarySubtype::Generic, bytes: vec![1, 2, 3] },
     ///     "bool": true,
-    /// };
+    /// })?;
     ///
     /// assert_eq!(&doc.get_binary("binary")?.bytes, &[1, 2, 3]);
     /// assert!(matches!(doc.get_binary("bool").unwrap_err().kind, ValueAccessErrorKind::UnexpectedType { .. }));
     /// assert!(matches!(doc.get_binary("unknown").unwrap_err().kind, ValueAccessErrorKind::NotPresent));
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
-    pub fn get_binary(&self, key: impl AsRef<str>) -> ValueAccessResult<RawBinaryRef<'_>> {
-        self.get_with(key, ElementType::Binary, RawBsonRef::as_binary)
+    pub fn get_binary(&self, key: impl AsRef<str>) -> ValueAccessResult<RawBinary<'_>> {
+        self.get_with(key, ElementType::Binary, RawBson::as_binary)
     }
 
     /// Gets a reference to the ObjectId value corresponding to a given key or returns an error if
@@ -326,12 +326,12 @@ impl RawDocument {
     ///
     /// ```
     /// # use bson::raw::Error;
-    /// use bson::{rawdoc, oid::ObjectId, raw::ValueAccessErrorKind};
+    /// use bson::{doc, oid::ObjectId, raw::{ValueAccessErrorKind, RawDocumentBuf}};
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "_id": ObjectId::new(),
     ///     "bool": true,
-    /// };
+    /// })?;
     ///
     /// let oid = doc.get_object_id("_id")?;
     /// assert!(matches!(doc.get_object_id("bool").unwrap_err().kind, ValueAccessErrorKind::UnexpectedType { .. }));
@@ -339,7 +339,7 @@ impl RawDocument {
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn get_object_id(&self, key: impl AsRef<str>) -> ValueAccessResult<ObjectId> {
-        self.get_with(key, ElementType::ObjectId, RawBsonRef::as_object_id)
+        self.get_with(key, ElementType::ObjectId, RawBson::as_object_id)
     }
 
     /// Gets a reference to the boolean value corresponding to a given key or returns an error if
@@ -347,12 +347,12 @@ impl RawDocument {
     ///
     /// ```
     /// # use bson::raw::Error;
-    /// use bson::{rawdoc, oid::ObjectId, raw::ValueAccessErrorKind};
+    /// use bson::{doc, oid::ObjectId, raw::{RawDocumentBuf, ValueAccessErrorKind}};
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "_id": ObjectId::new(),
     ///     "bool": true,
-    /// };
+    /// })?;
     ///
     /// assert!(doc.get_bool("bool")?);
     /// assert!(matches!(doc.get_bool("_id").unwrap_err().kind, ValueAccessErrorKind::UnexpectedType { .. }));
@@ -360,7 +360,7 @@ impl RawDocument {
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn get_bool(&self, key: impl AsRef<str>) -> ValueAccessResult<bool> {
-        self.get_with(key, ElementType::Boolean, RawBsonRef::as_bool)
+        self.get_with(key, ElementType::Boolean, RawBson::as_bool)
     }
 
     /// Gets a reference to the BSON DateTime value corresponding to a given key or returns an
@@ -368,13 +368,13 @@ impl RawDocument {
     ///
     /// ```
     /// # use bson::raw::Error;
-    /// use bson::{rawdoc, raw::ValueAccessErrorKind, DateTime};
+    /// use bson::{doc, raw::{ValueAccessErrorKind, RawDocumentBuf}, DateTime};
     ///
     /// let dt = DateTime::now();
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "created_at": dt,
     ///     "bool": true,
-    /// };
+    /// })?;
     ///
     /// assert_eq!(doc.get_datetime("created_at")?, dt);
     /// assert!(matches!(doc.get_datetime("bool").unwrap_err().kind, ValueAccessErrorKind::UnexpectedType { .. }));
@@ -382,31 +382,31 @@ impl RawDocument {
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn get_datetime(&self, key: impl AsRef<str>) -> ValueAccessResult<DateTime> {
-        self.get_with(key, ElementType::DateTime, RawBsonRef::as_datetime)
+        self.get_with(key, ElementType::DateTime, RawBson::as_datetime)
     }
 
     /// Gets a reference to the BSON regex value corresponding to a given key or returns an error if
     /// the key corresponds to a value which isn't a regex.
     ///
     /// ```
-    /// use bson::{rawdoc, Regex, raw::ValueAccessErrorKind};
+    /// use bson::{doc, Regex, raw::{RawDocumentBuf, ValueAccessErrorKind}};
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "regex": Regex {
     ///         pattern: r"end\s*$".into(),
     ///         options: "i".into(),
     ///     },
     ///     "bool": true,
-    /// };
+    /// })?;
     ///
-    /// assert_eq!(doc.get_regex("regex")?.pattern, r"end\s*$");
-    /// assert_eq!(doc.get_regex("regex")?.options, "i");
+    /// assert_eq!(doc.get_regex("regex")?.pattern(), r"end\s*$");
+    /// assert_eq!(doc.get_regex("regex")?.options(), "i");
     /// assert!(matches!(doc.get_regex("bool").unwrap_err().kind, ValueAccessErrorKind::UnexpectedType { .. }));
     /// assert!(matches!(doc.get_regex("unknown").unwrap_err().kind, ValueAccessErrorKind::NotPresent));
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
-    pub fn get_regex(&self, key: impl AsRef<str>) -> ValueAccessResult<RawRegexRef<'_>> {
-        self.get_with(key, ElementType::RegularExpression, RawBsonRef::as_regex)
+    pub fn get_regex(&self, key: impl AsRef<str>) -> ValueAccessResult<RawRegex<'_>> {
+        self.get_with(key, ElementType::RegularExpression, RawBson::as_regex)
     }
 
     /// Gets a reference to the BSON timestamp value corresponding to a given key or returns an
@@ -414,12 +414,12 @@ impl RawDocument {
     ///
     /// ```
     /// # use bson::raw::Error;
-    /// use bson::{rawdoc, Timestamp, raw::ValueAccessErrorKind};
+    /// use bson::{doc, Timestamp, raw::{RawDocumentBuf, ValueAccessErrorKind}};
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "bool": true,
     ///     "ts": Timestamp { time: 649876543, increment: 9 },
-    /// };
+    /// })?;
     ///
     /// let timestamp = doc.get_timestamp("ts")?;
     ///
@@ -430,7 +430,7 @@ impl RawDocument {
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn get_timestamp(&self, key: impl AsRef<str>) -> ValueAccessResult<Timestamp> {
-        self.get_with(key, ElementType::Timestamp, RawBsonRef::as_timestamp)
+        self.get_with(key, ElementType::Timestamp, RawBson::as_timestamp)
     }
 
     /// Gets a reference to the BSON int32 value corresponding to a given key or returns an error if
@@ -438,12 +438,12 @@ impl RawDocument {
     ///
     /// ```
     /// # use bson::raw::Error;
-    /// use bson::{rawdoc, raw::ValueAccessErrorKind};
+    /// use bson::{doc, raw::{RawDocumentBuf, ValueAccessErrorKind}};
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "bool": true,
     ///     "i32": 1_000_000,
-    /// };
+    /// })?;
     ///
     /// assert_eq!(doc.get_i32("i32")?, 1_000_000);
     /// assert!(matches!(doc.get_i32("bool").unwrap_err().kind, ValueAccessErrorKind::UnexpectedType { ..}));
@@ -451,7 +451,7 @@ impl RawDocument {
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn get_i32(&self, key: impl AsRef<str>) -> ValueAccessResult<i32> {
-        self.get_with(key, ElementType::Int32, RawBsonRef::as_i32)
+        self.get_with(key, ElementType::Int32, RawBson::as_i32)
     }
 
     /// Gets a reference to the BSON int64 value corresponding to a given key or returns an error if
@@ -459,12 +459,12 @@ impl RawDocument {
     ///
     /// ```
     /// # use bson::raw::Error;
-    /// use bson::{rawdoc, raw::ValueAccessErrorKind};
+    /// use bson::{doc, raw::{ValueAccessErrorKind, RawDocumentBuf}};
     ///
-    /// let doc = rawdoc! {
+    /// let doc = RawDocumentBuf::from_document(&doc! {
     ///     "bool": true,
     ///     "i64": 9223372036854775807_i64,
-    /// };
+    /// })?;
     ///
     /// assert_eq!(doc.get_i64("i64")?, 9223372036854775807);
     /// assert!(matches!(doc.get_i64("bool").unwrap_err().kind, ValueAccessErrorKind::UnexpectedType { .. }));
@@ -472,26 +472,21 @@ impl RawDocument {
     /// # Ok::<(), Box<dyn std::error::Error>>(())
     /// ```
     pub fn get_i64(&self, key: impl AsRef<str>) -> ValueAccessResult<i64> {
-        self.get_with(key, ElementType::Int64, RawBsonRef::as_i64)
+        self.get_with(key, ElementType::Int64, RawBson::as_i64)
     }
 
     /// Return a reference to the contained data as a `&[u8]`
     ///
     /// ```
     /// # use bson::raw::Error;
-    /// use bson::rawdoc;
-    /// let docbuf = rawdoc! {};
+    /// use bson::{doc, raw::RawDocumentBuf};
+    /// let docbuf = RawDocumentBuf::from_document(&doc!{})?;
     /// assert_eq!(docbuf.as_bytes(), b"\x05\x00\x00\x00\x00");
     /// # Ok::<(), Error>(())
     /// ```
     pub fn as_bytes(&self) -> &[u8] {
         &self.data
     }
-
-    /// Returns whether this document contains any elements or not.
-    pub fn is_empty(&self) -> bool {
-        self.as_bytes().len() == MIN_BSON_DOCUMENT_SIZE as usize
-    }
 }
 
 impl<'de: 'a, 'a> Deserialize<'de> for &'a RawDocument {
@@ -499,11 +494,19 @@ impl<'de: 'a, 'a> Deserialize<'de> for &'a RawDocument {
     where
         D: serde::Deserializer<'de>,
     {
-        match OwnedOrBorrowedRawDocument::deserialize(deserializer)? {
-            OwnedOrBorrowedRawDocument::Borrowed(b) => Ok(b),
-            OwnedOrBorrowedRawDocument::Owned(d) => Err(serde::de::Error::custom(format!(
-                "expected borrowed raw document, instead got owned {:?}",
-                d
+        match deserializer.deserialize_newtype_struct(RAW_DOCUMENT_NEWTYPE, RawBsonVisitor)? {
+            RawBson::Document(d) => Ok(d),
+
+            // For non-BSON formats, RawDocument gets serialized as bytes, so we need to deserialize
+            // from them here too. For BSON, the deserializier will return an error if it
+            // sees the RAW_DOCUMENT_NEWTYPE but the next type isn't a document.
+            RawBson::Binary(b) if b.subtype == BinarySubtype::Generic => {
+                RawDocument::new(b.bytes).map_err(serde::de::Error::custom)
+            }
+
+            o => Err(serde::de::Error::custom(format!(
+                "expected raw document reference, instead got {:?}",
+                o
             ))),
         }
     }
@@ -578,7 +581,7 @@ impl TryFrom<&RawDocument> for crate::Document {
 
 impl<'a> IntoIterator for &'a RawDocument {
     type IntoIter = Iter<'a>;
-    type Item = Result<(&'a str, RawBsonRef<'a>)>;
+    type Item = Result<(&'a str, RawBson<'a>)>;
 
     fn into_iter(self) -> Iter<'a> {
         Iter::new(self)
